@@ -48,6 +48,7 @@ static char const * auto_detect_format(sox_format_t * ft, char const * ext)
   CHECK(voc   , 0, 0, ""     , 0, 20, "Creative Voice File\x1a")
   CHECK(smp   , 0, 0, ""     , 0, 17, "SOUND SAMPLE DATA")
   CHECK(wve   , 0, 0, ""     , 0, 15, "ALawSoundFile**")
+  CHECK(gsrt  , 0, 0, ""     , 16, 9, "ring.bin")
   CHECK(amr-wb, 0, 0, ""     , 0,  9, "#!AMR-WB\n")
   CHECK(prc   , 0, 0, ""     , 0,  8, "\x37\x00\x00\x10\x6d\x00\x00\x10")
   CHECK(sph   , 0, 0, ""     , 0,  7, "NIST_1A")
@@ -170,8 +171,9 @@ unsigned sox_precision(sox_encoding_t encoding, unsigned bits_per_sample)
     case SOX_ENCODING_CVSD:       return bits_per_sample == 1? 16: 0;
     case SOX_ENCODING_DPCM:       return bits_per_sample; /* ? */
 
+    case SOX_ENCODING_MP3:        return 0; /* Accept the precision returned by the format. */
+
     case SOX_ENCODING_GSM:
-    case SOX_ENCODING_MP3:
     case SOX_ENCODING_VORBIS:
     case SOX_ENCODING_AMR_WB:
     case SOX_ENCODING_AMR_NB:
@@ -392,7 +394,9 @@ static FILE * xfopen(char const * identifier, char const * mode, lsx_io_type * i
  * Works by resetting the FILE buffer pointer */
 static void UNUSED rewind_pipe(FILE * fp)
 {
-#if defined _NEWLIB_VERSION || defined __APPLE__
+/* _FSTDIO is for Torek stdio (i.e. most BSD-derived libc's)
+ * In theory, we no longer need to check _NEWLIB_VERSION or __APPLE__ */
+#if defined _FSTDIO || defined _NEWLIB_VERSION || defined __APPLE__
   fp->_p -= AUTO_DETECT_SIZE;
   fp->_r += AUTO_DETECT_SIZE;
 #elif defined __GLIBC__
@@ -409,8 +413,10 @@ static void UNUSED rewind_pipe(FILE * fp)
 #endif
 }
 
-sox_format_t * sox_open_read(
+static sox_format_t * open_read(
     char               const * path,
+    void                     * buffer UNUSED,
+    size_t                     buffer_size UNUSED,
     sox_signalinfo_t   const * signal,
     sox_encodinginfo_t const * encoding,
     char               const * filetype)
@@ -441,7 +447,11 @@ sox_format_t * sox_open_read(
       ft->fp = stdin;
     }
     else {
-      ft->fp = xfopen(path, "rb", &ft->io_type);
+      ft->fp =
+#ifdef HAVE_FMEMOPEN
+        buffer? fmemopen(buffer, buffer_size, "rb") :
+#endif
+        xfopen(path, "rb", &ft->io_type);
       type = io_types[ft->io_type];
       if (ft->fp == NULL) {
         lsx_fail("can't open input %s `%s': %s", type, path, strerror(errno));
@@ -551,6 +561,25 @@ error:
   return NULL;
 }
 
+sox_format_t * sox_open_read(
+    char               const * path,
+    sox_signalinfo_t   const * signal,
+    sox_encodinginfo_t const * encoding,
+    char               const * filetype)
+{
+  return open_read(path, NULL, (size_t)0, signal, encoding, filetype);
+}
+
+sox_format_t * sox_open_mem_read(
+    void                     * buffer,
+    size_t                     buffer_size,
+    sox_signalinfo_t   const * signal,
+    sox_encodinginfo_t const * encoding,
+    char               const * filetype)
+{
+  return open_read("", buffer, buffer_size, signal,encoding,filetype);
+}
+
 sox_bool sox_format_supports_encoding(
     char               const * path,
     char               const * filetype,
@@ -773,8 +802,12 @@ sox_format_handler_t const * sox_write_handler(
   return handler;
 }
 
-sox_format_t * sox_open_write(
+static sox_format_t * open_write(
     char               const * path,
+    void                     * buffer UNUSED,
+    size_t                     buffer_size UNUSED,
+    char                     * * buffer_ptr UNUSED,
+    size_t                   * buffer_size_ptr UNUSED,
     sox_signalinfo_t   const * signal,
     sox_encodinginfo_t const * encoding,
     char               const * filetype,
@@ -811,7 +844,13 @@ sox_format_t * sox_open_write(
         lsx_fail("permission to overwrite `%s' denied", path);
         goto error;
       }
-      if ((ft->fp = fopen(path, "wb")) == NULL) {
+      ft->fp =
+#ifdef HAVE_FMEMOPEN
+        buffer? fmemopen(buffer, buffer_size, "w+b") :
+        buffer_ptr? open_memstream(buffer_ptr, buffer_size_ptr) :
+#endif
+        fopen(path, "w+b");
+      if (ft->fp == NULL) {
         lsx_fail("can't open output file `%s': %s", path, strerror(errno));
         goto error;
       }
@@ -883,6 +922,39 @@ error:
   return NULL;
 }
 
+sox_format_t * sox_open_write(
+    char               const * path,
+    sox_signalinfo_t   const * signal,
+    sox_encodinginfo_t const * encoding,
+    char               const * filetype,
+    sox_oob_t          const * oob,
+    sox_bool           (*overwrite_permitted)(const char *filename))
+{
+  return open_write(path, NULL, (size_t)0, NULL, NULL, signal, encoding, filetype, oob, overwrite_permitted);
+}
+
+sox_format_t * sox_open_mem_write(
+    void                     * buffer,
+    size_t                     buffer_size,
+    sox_signalinfo_t   const * signal,
+    sox_encodinginfo_t const * encoding,
+    char               const * filetype,
+    sox_oob_t          const * oob)
+{
+  return open_write("", buffer, buffer_size, NULL, NULL, signal, encoding, filetype, oob, NULL);
+}
+
+sox_format_t * sox_open_memstream_write(
+    char                     * * buffer_ptr,
+    size_t                   * buffer_size_ptr,
+    sox_signalinfo_t   const * signal,
+    sox_encodinginfo_t const * encoding,
+    char               const * filetype,
+    sox_oob_t          const * oob)
+{
+  return open_write("", NULL, (size_t)0, buffer_ptr, buffer_size_ptr, signal, encoding, filetype, oob, NULL);
+}
+
 size_t sox_read(sox_format_t * ft, sox_sample_t * buf, size_t len)
 {
   size_t actual;
@@ -921,6 +993,7 @@ int sox_close(sox_format_t * ft)
 
   if (ft->fp && ft->fp != stdin && ft->fp != stdout)
     xfclose(ft->fp, ft->io_type);
+  free(ft->priv);
   free(ft->filename);
   free(ft->filetype);
   sox_delete_comments(&ft->oob.comments);
@@ -1073,7 +1146,6 @@ enum {
 static sox_bool plugins_initted = sox_false;
 
 #ifdef HAVE_LIBLTDL /* Plugin format handlers */
-  #include <ltdl.h>
   #define MAX_DYNAMIC_FORMATS 42
   #define MAX_FORMATS (NSTATIC_FORMATS + MAX_DYNAMIC_FORMATS)
   #define MAX_FORMATS_1 (MAX_FORMATS + 1)
