@@ -285,7 +285,7 @@ static sox_bool aboveThreshold(sox_effect_t const * effp,
   else if (unit == 'd')
     scaled_value = linear_to_dB(scaled_value);
 
-  return scaled_value >= threshold;
+  return scaled_value > threshold;
 }
 
 static sox_sample_t compute_rms(sox_effect_t * effp, sox_sample_t sample)
@@ -652,7 +652,7 @@ static int sox_silence_stop(sox_effect_t * effp)
   return(SOX_SUCCESS);
 }
 
-static int kill(sox_effect_t * effp)
+static int lsx_kill(sox_effect_t * effp)
 {
   priv_t * silence = (priv_t *) effp->priv;
 
@@ -665,13 +665,13 @@ static int kill(sox_effect_t * effp)
 static sox_effect_handler_t sox_silence_effect = {
   "silence",
   "[ -l ] above_periods [ duration thershold[d|%%] ] [ below_periods duration threshold[d|%%]]",
-  SOX_EFF_MCHAN | SOX_EFF_MODIFY,
+  SOX_EFF_MCHAN | SOX_EFF_MODIFY | SOX_EFF_LENGTH,
   sox_silence_getopts,
   sox_silence_start,
   sox_silence_flow,
   sox_silence_drain,
   sox_silence_stop,
-  kill, sizeof(priv_t)
+  lsx_kill, sizeof(priv_t)
 };
 
 const sox_effect_handler_t *lsx_silence_effect_fn(void)
