@@ -38,7 +38,10 @@
 #include <sys/types.h>
 #include <time.h>
 
-#ifdef HAVE_GLOB_H
+#if defined(HAVE_WIN32_GLOB_H)
+  #include "win32-glob.h"
+  #define HAVE_GLOB_H 1
+#elif defined(HAVE_GLOB_H)
   #include <glob.h>
 #endif
 
@@ -92,6 +95,16 @@
   #define TIME_FRAC 1e3
 #endif
 
+#if !defined(HAVE_CONIO_H) && !defined(HAVE_TERMIOS_H) && (defined(_MSC_VER) || defined(__MINGW32__))
+#define HAVE_CONIO_H 1
+#endif
+
+#ifdef HAVE_CONIO_H
+/* _kbhit and _getch */
+#include <conio.h>
+#undef HAVE_TERMIOS_H
+#endif
+
 /*#define MORE_INTERACTIVE 1*/
 
 /* argv[0] options */
@@ -190,7 +203,7 @@ static sox_sample_t omax[2], omin[2];
 
 /* Multi-processing */
 
-static sox_bool single_threaded = sox_false;
+static sox_bool single_threaded = sox_true;
 
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
@@ -235,7 +248,7 @@ static void cleanup(void)
     tcsetattr(fileno(stdin), TCSANOW, &original_termios);
 #endif
 
-  sox_format_quit();
+  sox_quit();
 }
 
 static char const * str_time(double seconds)
@@ -539,7 +552,7 @@ static int combiner_drain(sox_effect_t *effp, sox_sample_t * obuf, size_t * osam
             }
         }
       } /* sox_mix */ else if (combine_method == sox_multiply)  {
-        for (s = 0; s < effp->in_signal.channels; ++s, ++p) { /* multiple samples */
+        for (s = 0; s < effp->in_signal.channels; ++s, ++p) { /* multiply samples */
           i = 0;
           *p = ws < z->ilen[i] && s < files[i]->ft->signal.channels?
             z->ibuf[i][ws * files[i]->ft->signal.channels + s] : 0;
@@ -1167,7 +1180,7 @@ static int kbhit(void)
   select(fileno(stdin) + 1, &fdset, NULL, NULL, &time_val);
   return FD_ISSET(fileno(stdin), &fdset);
 }
-#elif !defined(_MSC_VER)
+#elif !defined(HAVE_CONIO_H)
 #define kbhit() 0
 #endif
 
@@ -1224,7 +1237,11 @@ static int update_status(sox_bool all_done, void * client_data)
 {
   (void)client_data;
   if (interactive) while (kbhit()) {
+#ifdef HAVE_CONIO_H
+    int ch = _getch();
+#else
     int ch = getchar();
+#endif
 
 #ifdef MORE_INTERACTIVE
     if (files[current_input]->ft->handler.seek &&
@@ -1348,34 +1365,28 @@ static char *fndup_with_count(const char *filename, size_t count)
          */
         if (*fn == '%')
         {
-            int width = 0;
+            char width = 0;
             fn++;
-            if (*fn >= '1' || *fn <= '9')
+            if (*fn >= '1' && *fn <= '9')
             {
                 width = *fn++;
             }
             if (*fn == 'n')
             {
-                char num[10];
                 char format[5];
 
                 found_marker = sox_true;
 
-                strcpy(format, "%");
                 if (width)
                 {
-                    char tmps[2];
-                    tmps[0] = width;
-                    tmps[1] = 0;
-                    strcat(format, "0");
-                    strcat(format, tmps);
+					sprintf(format, "%%0%cd", width);
                 }
-                strcat(format, "d");
-                strcpy(format, "%02d");
-                sprintf(num, format, count);
-                *efn = 0;
-                strcat(efn, num);
-                efn += strlen(num);
+				else
+				{
+                    strcpy(format, "%02d");
+				}
+
+                efn += sprintf(efn, format, count);
                 fn++;
             }
             else
@@ -1465,7 +1476,10 @@ static void sigint(int s)
   static struct timeval then;
   if (input_count > 1 && show_progress && s == SIGINT &&
       is_serial(combine_method) && since(&then, 1.0, sox_true))
+  {
+    signal(SIGINT, sigint);
     user_skip = sox_true;
+  }
   else user_abort = sox_true;
 }
 
@@ -1627,16 +1641,19 @@ static int process(void)
 
   optimize_trim();
 
-#ifdef HAVE_TERMIOS_H /* so we can be fully interactive. */
-  if (!interactive && is_player && stdin_is_a_tty) {
+#if defined(HAVE_TERMIOS_H) || defined(HAVE_CONIO_H)
+  /* so we can be fully interactive. */
+  if (show_progress && !interactive && is_player && stdin_is_a_tty) {
+#ifdef HAVE_TERMIOS_H
     struct termios modified_termios;
 
     tcgetattr(fileno(stdin), &original_termios);
-    interactive = sox_true;
     modified_termios = original_termios;
     modified_termios.c_lflag &= ~(ICANON | ECHO);
     modified_termios.c_cc[VMIN] = modified_termios.c_cc[VTIME] = 0;
     tcsetattr(fileno(stdin), TCSANOW, &modified_termios);
+#endif
+    interactive = sox_true;
   }
 #endif
 
@@ -1724,7 +1741,7 @@ static void display_supported_formats(void)
         if (!strchr(*names, '/'))
           format_list[formats++] = *names;
   }
-  qsort(format_list, formats, sizeof(*format_list), strcmp_p);
+  qsort((void*)format_list, formats, sizeof(*format_list), strcmp_p);
   for (i = 0; i < formats; i++)
     printf(" %s", format_list[i]);
   putchar('\n');
@@ -1736,12 +1753,12 @@ static void display_supported_formats(void)
       for (names = handler->names; *names; ++names)
         format_list[formats++] = *names;
   }
-  qsort(format_list, formats, sizeof(*format_list), strcmp_p);
+  qsort((void*)format_list, formats, sizeof(*format_list), strcmp_p);
   for (i = 0; i < formats; i++)
     printf(" %s", format_list[i]);
   puts("\n");
 
-  free(format_list);
+  free((void*)format_list);
 }
 
 static void display_supported_effects(void)
@@ -1761,7 +1778,7 @@ static void display_supported_effects(void)
 static void usage(char const * message)
 {
   size_t i;
-  static char const * lines[] = {
+  static char const * const lines[] = {
 "SPECIAL FILENAMES (infile, outfile):",
 "-                        Pipe/redirect input/output (stdin/stdout); may need -t",
 "-d, --default-device     Use the default audio device (where available)",
@@ -1776,8 +1793,8 @@ static void usage(char const * message)
 "GLOBAL OPTIONS (gopts) (can be specified at any point before the first effect):",
 "--buffer BYTES           Set the size of all processing buffers (default 8192)",
 "--clobber                Don't prompt to overwrite output file (default)",
-"--combine concatenate    Concatenate multiple input files (default for sox, rec)",
-"--combine sequence       Sequence multiple input files (default for play)",
+"--combine concatenate    Concatenate all input files (default for sox, rec)",
+"--combine sequence       Sequence all input files (default for play)",
 "-D, --no-dither          Don't dither automatically",
 "--effects-file FILENAME  File containing effects and options",
 "-G, --guard              Use temporary files to guard against clipping",
@@ -1788,8 +1805,11 @@ static void usage(char const * message)
 "--input-buffer BYTES     Override the input buffer size (default: as --buffer)",
 "--no-clobber             Prompt to overwrite output file",
 "-m, --combine mix        Mix multiple input files (instead of concatenating)",
+"--combine mix-power      Mix to equal power (instead of concatenating)",
 "-M, --combine merge      Merge multiple input files (instead of concatenating)",
 "--magic                  Use `magic' file-type detection",
+"--multi-threaded         Enable parallel effects channels processing (where",
+"                         available)",
 "--norm                   Guard (see --guard) & normalise",
 "--play-rate-arg ARG      Default `rate' argument for auto-resample with `play'",
 "--plot gnuplot|octave    Generate script to plot response of filter effect",
@@ -1799,6 +1819,8 @@ static void usage(char const * message)
 "-S, --show-progress      Display progress while processing audio data",
 "--single-threaded        Disable parallel effects channels processing",
 "--temp DIRECTORY         Specify the directory to use for temporary files",
+"-T, --combine multiply   Multiply samples of corresponding channels from all",
+"                         input files (instead of concatenating)",
 "--version                Display version number of SoX and exit",
 "-V[LEVEL]                Increment or set verbosity level (default 2); levels:",
 "                           1: failure messages",
@@ -1813,7 +1835,7 @@ static void usage(char const * message)
 "-v|--volume FACTOR       Input file volume adjustment factor (real number)",
 "--ignore-length          Ignore input file length given in header; read to EOF",
 "-t|--type FILETYPE       File type of audio",
-"-s/-u/-f/-U/-A/-i/-a/-g  Encoding type=signed-integer/unsigned-integer/floating-",
+"-s/-u/-f/-U/-A/-i/-a/-g  Encoding type=signed-integer/unsigned-integer/floating",
 "                         point/mu-law/a-law/ima-adpcm/ms-adpcm/gsm-full-rate",
 "-e|--encoding ENCODING   Set encoding (ENCODING in above list)",
 "-b|--bits BITS           Encoded sample size in bits",
@@ -1839,7 +1861,7 @@ static void usage(char const * message)
   if (message)
     lsx_fail("%s\n", message);  /* N.B. stderr */
 
-  printf("Usage summary: [gopts] [[fopts] infile]... [fopts]%s [effect [effopts]]...\n\n",
+  printf("Usage summary: [gopts] [[fopts] infile]... [fopts]%s [effect [effopt]]...\n\n",
          sox_mode == sox_play? "" : " outfile");
   for (i = 0; i < array_length(lines); ++i)
     puts(lines[i]);
@@ -1989,55 +2011,56 @@ static void read_comment_file(sox_comments_t * comments, char const * const file
   free(text);
 }
 
-static char *getoptstr = "+ab:c:de:fghimnopqr:st:uv:xABC:DGLMNRSTUV::X12348";
+static char const * const getoptstr =
+  "+ab:c:de:fghimnopqr:st:uv:xABC:DGLMNRSTUV::X12348";
 
-static struct option long_options[] =
-  {
-    {"add-comment"     , required_argument, NULL, 0},
-    {"buffer"          , required_argument, NULL, 0},
-    {"combine"         , required_argument, NULL, 0},
-    {"comment-file"    , required_argument, NULL, 0},
-    {"comment"         , required_argument, NULL, 0},
-    {"endian"          , required_argument, NULL, 0},
-    {"input-buffer"    , required_argument, NULL, 0},
-    {"interactive"     ,       no_argument, NULL, 0},
-    {"help-effect"     , required_argument, NULL, 0},
-    {"help-format"     , required_argument, NULL, 0},
-    {"no-glob"         ,       no_argument, NULL, 0},
-    {"plot"            , required_argument, NULL, 0},
-    {"replay-gain"     , required_argument, NULL, 0},
-    {"version"         ,       no_argument, NULL, 0},
-    {"output"          , required_argument, NULL, 0},
-    {"effects-file"    , required_argument, NULL, 0},
-    {"temp"            , required_argument, NULL, 0},
-    {"single-threaded" ,       no_argument, NULL, 0},
-    {"ignore-length"   ,       no_argument, NULL, 0},
-    {"norm"            ,       no_argument, NULL, 0},
-    {"magic"           ,       no_argument, NULL, 0},
-    {"play-rate-arg"   , required_argument, NULL, 0},
-    {"clobber"         ,       no_argument, NULL, 0},
-    {"no-clobber"      ,       no_argument, NULL, 0},
+static struct option const long_options[] = {
+  {"add-comment"     , required_argument, NULL, 0},
+  {"buffer"          , required_argument, NULL, 0},
+  {"combine"         , required_argument, NULL, 0},
+  {"comment-file"    , required_argument, NULL, 0},
+  {"comment"         , required_argument, NULL, 0},
+  {"endian"          , required_argument, NULL, 0},
+  {"input-buffer"    , required_argument, NULL, 0},
+  {"interactive"     ,       no_argument, NULL, 0},
+  {"help-effect"     , required_argument, NULL, 0},
+  {"help-format"     , required_argument, NULL, 0},
+  {"no-glob"         ,       no_argument, NULL, 0},
+  {"plot"            , required_argument, NULL, 0},
+  {"replay-gain"     , required_argument, NULL, 0},
+  {"version"         ,       no_argument, NULL, 0},
+  {"output"          , required_argument, NULL, 0},
+  {"effects-file"    , required_argument, NULL, 0},
+  {"temp"            , required_argument, NULL, 0},
+  {"single-threaded" ,       no_argument, NULL, 0},
+  {"ignore-length"   ,       no_argument, NULL, 0},
+  {"norm"            ,       no_argument, NULL, 0},
+  {"magic"           ,       no_argument, NULL, 0},
+  {"play-rate-arg"   , required_argument, NULL, 0},
+  {"clobber"         ,       no_argument, NULL, 0},
+  {"no-clobber"      ,       no_argument, NULL, 0},
+  {"multi-threaded"  ,       no_argument, NULL, 0},
 
-    {"bits"            , required_argument, NULL, 'b'},
-    {"channels"        , required_argument, NULL, 'c'},
-    {"compression"     , required_argument, NULL, 'C'},
-    {"default-device"  ,       no_argument, NULL, 'd'},
-    {"no-dither"       ,       no_argument, NULL, 'D'},
-    {"encoding"        , required_argument, NULL, 'e'},
-    {"help"            ,       no_argument, NULL, 'h'},
-    {"null"            ,       no_argument, NULL, 'n'},
-    {"no-show-progress",       no_argument, NULL, 'q'},
-    {"pipe"            ,       no_argument, NULL, 'p'},
-    {"rate"            , required_argument, NULL, 'r'},
-    {"reverse-bits"    ,       no_argument, NULL, 'X'},
-    {"reverse-nibbles" ,       no_argument, NULL, 'N'},
-    {"show-progress"   ,       no_argument, NULL, 'S'},
-    {"type"            , required_argument, NULL, 't'},
-    {"volume"          , required_argument, NULL, 'v'},
-    {"guard"           ,       no_argument, NULL, 'G'},
+  {"bits"            , required_argument, NULL, 'b'},
+  {"channels"        , required_argument, NULL, 'c'},
+  {"compression"     , required_argument, NULL, 'C'},
+  {"default-device"  ,       no_argument, NULL, 'd'},
+  {"no-dither"       ,       no_argument, NULL, 'D'},
+  {"encoding"        , required_argument, NULL, 'e'},
+  {"help"            ,       no_argument, NULL, 'h'},
+  {"null"            ,       no_argument, NULL, 'n'},
+  {"no-show-progress",       no_argument, NULL, 'q'},
+  {"pipe"            ,       no_argument, NULL, 'p'},
+  {"rate"            , required_argument, NULL, 'r'},
+  {"reverse-bits"    ,       no_argument, NULL, 'X'},
+  {"reverse-nibbles" ,       no_argument, NULL, 'N'},
+  {"show-progress"   ,       no_argument, NULL, 'S'},
+  {"type"            , required_argument, NULL, 't'},
+  {"volume"          , required_argument, NULL, 'v'},
+  {"guard"           ,       no_argument, NULL, 'G'},
 
-    {NULL, 0, NULL, 0}
-  };
+  {NULL, 0, NULL, 0}
+};
 
 static int opt_index(int val)
 {
@@ -2191,6 +2214,7 @@ static char parse_gopts_and_fopts(file_t * f, int argc, char **argv)
       case 21: play_rate_arg = strdup(lsx_optarg); break;
       case 22: no_clobber = sox_false; break;
       case 23: no_clobber = sox_true; break;
+      case 24: single_threaded = sox_false; break;
       }
       break;
 
@@ -2353,7 +2377,8 @@ static char const * device_name(char const * const type)
   else if (!strcmp(type, "oss" ) || !strcmp(type, "ossdsp"))
     name = "/dev/dsp";
   else if (!strcmp(type, "alsa") || !strcmp(type, "ao") || !strcmp(type,"sndio")
-        || !strcmp(type, "coreaudio") || !strcmp(type, "pulseaudio"))
+        || !strcmp(type, "coreaudio") || !strcmp(type, "pulseaudio")
+	|| !strcmp(type, "waveaudio"))
     name = "default";
   
   return name? from_env? from_env : name : NULL;
@@ -2365,10 +2390,11 @@ static char const * set_default_device(file_t * f)
   if (!f->filetype) f->filetype = getenv("AUDIODRIVER");
   if (!f->filetype && sox_find_format("coreaudio", sox_false)) f->filetype = "coreaudio";
   if (!f->filetype && sox_find_format("alsa", sox_false)) f->filetype = "alsa";
+  if (!f->filetype && sox_find_format("waveaudio" , sox_false)) f->filetype = "waveaudio";
+  if (!f->filetype && sox_find_format("sndio", sox_false)) f->filetype = "sndio";
   if (!f->filetype && sox_find_format("oss" , sox_false)) f->filetype = "oss";
   if (!f->filetype && sox_find_format("pulseaudio" , sox_false)) f->filetype = "pulseaudio";
   if (!f->filetype && sox_find_format("sunau",sox_false)) f->filetype = "sunau";
-  if (!f->filetype && sox_find_format("sndio", sox_false)) f->filetype = "sndio";
   if (!f->filetype && sox_find_format("ao"  , sox_false) && file_count) /*!rec*/
     f->filetype = "ao";
 
@@ -2528,6 +2554,34 @@ static int soxi1(soxi_t const * type, char * filename)
   return !!sox_close(ft);
 }
 
+static void soxi_usage(int return_code)
+{
+  display_SoX_version(stdout);
+  printf(
+    "\n"
+    "Usage: soxi [-V[level]] [-T] [-t|-r|-c|-s|-d|-D|-b|-B|-e|-a] infile1 ...\n"
+    "\n"
+    "-V[n]\tIncrement or set verbosity level (default is 2)\n"
+    "-T\tWith -s, -d or -D, display the total across all given files\n"
+    "\n"
+    "-t\tShow detected file-type\n"
+    "-r\tShow sample-rate\n"
+    "-c\tShow number of channels\n"
+    ); printf(
+    "-s\tShow number of samples (0 if unavailable)\n"
+    "-d\tShow duration in hours, minutes and seconds (0 if unavailable)\n"
+    "-D\tShow duration in seconds (0 if unavailable)\n"
+    "-b\tShow number of bits per sample\n"
+    "-B\tShow the bitrate averaged over the whole file (0 if unavailable)\n"
+    "-e\tShow the name of the audio encoding\n"
+    "-a\tShow file comments (annotations) if available\n"
+    "\n"
+    "With no options, as much information as is available is shown for\n"
+    "each given file.\n"
+    );
+  exit(return_code);
+}
+
 static int soxi(int argc, char * const * argv)
 {
   static char const opts[] = "trcsdDbBea?TV::";
@@ -2535,6 +2589,8 @@ static int soxi(int argc, char * const * argv)
   int opt, num_errors = 0;
   sox_bool do_total = sox_false;
 
+  if (argc < 2)
+    soxi_usage(0);
   while ((opt = lsx_getopt(argc, argv, opts)) > 0) /* act only on last option */
     if (opt == 'V') {
       int i; /* sscanf silently accepts negative numbers for %u :( */
@@ -2552,10 +2608,9 @@ static int soxi(int argc, char * const * argv)
     }
     else if (opt == 'T')
       do_total = sox_true;
-    else if ((type = 1 + (strchr(opts, opt) - opts)) > Annotation) {
-      printf("Usage: soxi [-V[level]] [-T] [-t|-r|-c|-s|-d|-D|-b|-B|-e|-a] infile1 ...\n");
-      return -1;
-    }
+    else if ((type = 1 + (strchr(opts, opt) - opts)) > Annotation)
+      soxi_usage(1);
+
   if (type == Full)
     do_total = sox_true;
   else if (do_total && (type < Samples || type > Duration_secs)) {
@@ -2617,7 +2672,7 @@ static sox_bool cmp_comment_text(char const * c1, char const * c2)
   return c1 && c2 && !strcasecmp(c1, c2);
 }
 
-#ifdef __CYGWIN__
+#if defined(__CYGWIN__) || defined(__MINGW32__)
 static char * check_dir(char * name)
 {
   struct stat st;
@@ -2657,7 +2712,8 @@ int main(int argc, char **argv)
 
   parse_options_and_filenames(argc, argv);
 
-#ifdef __CYGWIN__  /* Workarounds for a couple of cygwin problems: */
+#if defined(__CYGWIN__) || defined(__MINGW32__) 
+  /* Workarounds for a couple of cygwin/mingw problems: */
   /* Allow command-line --temp "" to specify default behaviour: */
   if (sox_globals.tmp_path && !*sox_globals.tmp_path) {
     free(sox_globals.tmp_path);
@@ -2699,7 +2755,7 @@ int main(int argc, char **argv)
     combine_method = sox_concatenate;
 
   /* Make sure we got at least the required # of input filenames */
-  if (input_count < (is_serial(combine_method) ? 1 : 2))
+  if (input_count < (size_t)(is_serial(combine_method) ? 1 : 2))
     usage("Not enough input filenames specified");
 
   /* Check for misplaced input/output-specific options */
