@@ -725,7 +725,7 @@ static int stop(sox_effect_t * effp)
 
 
 
-static int kill(sox_effect_t * effp)
+static int lsx_kill(sox_effect_t * effp)
 {
   priv_t * p = (priv_t *) effp->priv;
   free(p->getopts_channels);
@@ -740,7 +740,7 @@ const sox_effect_handler_t *lsx_synth_effect_fn(void)
   static sox_effect_handler_t handler = {
     "synth", "[-j KEY] [-n] [length [offset [phase [p1 [p2 [p3]]]]]]] {type [combine] [[%]freq[k][:|+|/|-[%]freq2[k]] [offset [phase [p1 [p2 [p3]]]]]]}",
     SOX_EFF_MCHAN | SOX_EFF_LENGTH | SOX_EFF_GAIN,
-    getopts, start, flow, 0, stop, kill, sizeof(priv_t)
+    getopts, start, flow, 0, stop, lsx_kill, sizeof(priv_t)
   };
   return &handler;
 }
